@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -307,6 +307,14 @@ namespace OrthancPlugins
       OrthancPlugins::OrthancConfiguration servers;
       configuration_->GetSection(servers, "Servers");
       OrthancPlugins::DicomWebServers::GetInstance().Load(servers.GetJson());
+
+      // Check configuration during initialization
+      GetMetadataMode(Orthanc::ResourceType_Study);
+      GetMetadataMode(Orthanc::ResourceType_Series);
+
+      std::set<Orthanc::DicomTag> tags;
+      GetExtrapolatedMetadataTags(tags, Orthanc::ResourceType_Study);
+      GetExtrapolatedMetadataTags(tags, Orthanc::ResourceType_Series);
     }
 
 
@@ -326,6 +334,14 @@ namespace OrthancPlugins
     }
 
 
+    bool LookupBooleanValue(bool& target,
+                            const std::string& key)
+    {
+      assert(configuration_.get() != NULL);
+      return configuration_->LookupBooleanValue(target, key);
+    }
+
+
     unsigned int GetUnsignedIntegerValue(const std::string& key,
                                          unsigned int defaultValue)
     {
@@ -439,7 +455,7 @@ namespace OrthancPlugins
         if (boost::iequals(it->first, name))
         {
           value = it->second;
-          return false;
+          return true;
         }
       }
 
@@ -605,5 +621,95 @@ namespace OrthancPlugins
         return false;   // By default, return DICOM+JSON
       }
     }
+
+    
+    MetadataMode GetMetadataMode(Orthanc::ResourceType level)
+    {
+      static const std::string FULL = "Full";
+      static const std::string MAIN_DICOM_TAGS = "MainDicomTags";
+      static const std::string EXTRAPOLATE = "Extrapolate";
+      
+      std::string key;
+      switch (level)
+      {
+        case Orthanc::ResourceType_Study:
+          key = "StudiesMetadata";
+          break;
+
+        case Orthanc::ResourceType_Series:
+          key = "SeriesMetadata";
+          break;
+
+        default:
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
+      }
+
+      std::string value = GetStringValue(key, FULL);
+
+      if (value == FULL)
+      {
+        return MetadataMode_Full;
+      }
+      else if (value == MAIN_DICOM_TAGS)
+      {
+        return MetadataMode_MainDicomTags;
+      }
+      else if (value == EXTRAPOLATE)
+      {
+        return MetadataMode_Extrapolate;
+      }
+      else
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange,
+                                        "Bad value for option \"" + key +
+                                        "\": Should be either \"" + FULL + "\" or \"" +
+                                        MAIN_DICOM_TAGS + "\" or \"" + EXTRAPOLATE + "\"");
+      }
+    }
+
+
+    void GetSetOfTags(std::set<Orthanc::DicomTag>& tags,
+                      const std::string& key)
+    {
+      tags.clear();
+
+      std::list<std::string> s;
+      
+      if (configuration_->LookupListOfStrings(s, key, false))
+      {
+        for (std::list<std::string>::const_iterator it = s.begin(); it != s.end(); ++it)
+        {
+          OrthancPluginDictionaryEntry entry;
+          if (OrthancPluginLookupDictionary(GetGlobalContext(), &entry, it->c_str()) == OrthancPluginErrorCode_Success)
+          {
+            tags.insert(Orthanc::DicomTag(entry.group, entry.element));
+          }
+          else
+          {
+            throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange,
+                                            "Unknown DICOM tag in option \"" + key + "\" of DICOMweb: " + *it);
+          }
+        }
+      }
+    }
+
+
+    void GetExtrapolatedMetadataTags(std::set<Orthanc::DicomTag>& tags,
+                                     Orthanc::ResourceType level)
+    {
+      switch (level)
+      {
+        case Orthanc::ResourceType_Study:
+          GetSetOfTags(tags, "StudiesMetadataExtrapolatedTags");
+          break;
+
+        case Orthanc::ResourceType_Series:
+          GetSetOfTags(tags, "SeriesMetadataExtrapolatedTags");
+          break;
+
+        default:
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
+      }
+    }
   }
 }
