@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -45,6 +45,14 @@ namespace OrthancPlugins
   static const Orthanc::DicomTag DICOM_TAG_REFERENCED_SOP_CLASS_UID(0x0008, 0x1150);
   static const Orthanc::DicomTag DICOM_TAG_REFERENCED_SOP_INSTANCE_UID(0x0008, 0x1155);
 
+  enum MetadataMode
+  {
+    MetadataMode_Full,           // Read all the DICOM instances from the storage area
+    MetadataMode_MainDicomTags,  // Only use the Orthanc database (main DICOM tags only)
+    MetadataMode_Extrapolate     // Extrapolate user-specified tags from a few DICOM instances
+  };
+
+
   bool LookupHttpHeader(std::string& value,
                         const OrthancPluginHttpRequest* request,
                         const std::string& header);
@@ -84,12 +92,14 @@ namespace OrthancPlugins
   {
     void Initialize();
 
-    std::string GetStringValue(const std::string& key,
-                               const std::string& defaultValue);
-
+    bool HasKey(const std::string& key);
+    
     bool GetBooleanValue(const std::string& key,
                          bool defaultValue);
 
+    bool LookupBooleanValue(bool& target,
+                            const std::string& key);
+
     unsigned int GetUnsignedIntegerValue(const std::string& key,
                                          unsigned int defaultValue);
 
@@ -115,5 +125,13 @@ namespace OrthancPlugins
 
     // TODO => REMOVE
     bool IsXmlExpected(const OrthancPluginHttpRequest* request);
+
+    MetadataMode GetMetadataMode(Orthanc::ResourceType level);
+
+    void GetSetOfTags(std::set<Orthanc::DicomTag>& tags,
+                      const std::string& key);
+
+    void GetExtrapolatedMetadataTags(std::set<Orthanc::DicomTag>& tags,
+                                     Orthanc::ResourceType level);
   }
 }
