@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -119,6 +119,7 @@ namespace OrthancPlugins
 
 
   void DicomWebServers::ConfigureHttpClient(HttpClient& client,
+                                            std::map<std::string, std::string>& userProperties,
                                             const std::string& name,
                                             const std::string& uri)
   {
@@ -134,10 +135,22 @@ namespace OrthancPlugins
       client.SetCredentials(parameters.GetUsername(), parameters.GetPassword());
     }
 
+    if (!parameters.GetCertificateFile().empty())
+    {
+      client.SetCertificate(
+        parameters.GetCertificateFile(),
+        parameters.GetCertificateKeyFile(),
+        parameters.GetCertificateKeyPassword());
+    }
+
+    client.SetPkcs11(parameters.IsPkcs11Enabled());
+
     // By default, enable chunked transfers
     client.SetChunkedTransfersAllowed(
       parameters.GetBooleanUserProperty(HAS_CHUNKED_TRANSFERS, true));
-  }    
+
+    userProperties = parameters.GetUserProperties();
+  }
 
 
   void DicomWebServers::DeleteServer(const std::string& name)
