@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -20,7 +20,6 @@
 
 #include "DicomWebClient.h"
 #include "DicomWebServers.h"
-#include "GdcmParsedDicomFile.h"
 #include "QidoRs.h"
 #include "StowRs.h"
 #include "WadoRs.h"
@@ -376,7 +375,8 @@ void DeleteClient(OrthancPluginRestOutput* output,
     }
 
     OrthancPlugins::HttpClient client;
-    OrthancPlugins::DicomWebServers::GetInstance().ConfigureHttpClient(client, serverName, uri);
+    std::map<std::string, std::string> userProperties;
+    OrthancPlugins::DicomWebServers::GetInstance().ConfigureHttpClient(client, userProperties, serverName, uri);
     client.SetMethod(OrthancPluginHttpMethod_Delete);
     client.Execute();
 
@@ -388,85 +388,6 @@ void DeleteClient(OrthancPluginRestOutput* output,
 
 
 
-static void AnswerFrameRendered(OrthancPluginRestOutput* output,
-                                int frame,
-                                const OrthancPluginHttpRequest* request)
-{
-  OrthancPluginContext* context = OrthancPlugins::GetGlobalContext();
-
-  if (request->method != OrthancPluginHttpMethod_Get)
-  {
-    OrthancPluginSendMethodNotAllowed(context, output, "GET");
-  }
-  else
-  {
-    std::string instanceId;
-    if (LocateInstance(output, instanceId, request))
-    {
-      Orthanc::MimeType mime = Orthanc::MimeType_Jpeg;  // This is the default in DICOMweb
-      
-      for (uint32_t i = 0; i < request->headersCount; i++)
-      {
-        if (boost::iequals(request->headersKeys[i], "Accept") &&
-            !boost::iequals(request->headersValues[i], "*/*"))
-        {
-          try
-          {
-            // TODO - Support conversion to GIF
-        
-            mime = Orthanc::StringToMimeType(request->headersValues[i]);
-            if (mime != Orthanc::MimeType_Png &&
-                mime != Orthanc::MimeType_Jpeg)
-            {
-              throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
-            }
-          }
-          catch (Orthanc::OrthancException&)
-          {
-            LOG(ERROR) << "Unsupported MIME type in WADO-RS rendered frame: " << request->headersValues[i];
-            throw;
-          }
-        }
-      }
-
-      std::map<std::string, std::string> headers;
-      headers["Accept"] = Orthanc::EnumerationToString(mime);
-
-      // NB: In DICOMweb, the "frame" parameter is in the range [1..N], whereas
-      // Orthanc uses range [0..N-1], hence the "-1" below
-      OrthancPlugins::MemoryBuffer buffer;
-      if (buffer.RestApiGet("/instances/" + instanceId + "/frames/" +
-                            boost::lexical_cast<std::string>(frame - 1) + "/preview", headers, false))
-      {
-        OrthancPluginAnswerBuffer(context, output, buffer.GetData(),
-                                  buffer.GetSize(), Orthanc::EnumerationToString(mime));
-      }
-    }
-  }
-}
-
-
-void RetrieveInstanceRendered(OrthancPluginRestOutput* output,
-                              const char* url,
-                              const OrthancPluginHttpRequest* request)
-{
-  AnswerFrameRendered(output, 1 /* first frame */, request);
-}
-
-
-void RetrieveFrameRendered(OrthancPluginRestOutput* output,
-                           const char* url,
-                           const OrthancPluginHttpRequest* request)
-{
-  assert(request->groupsCount == 4);
-  const char* frame = request->groups[3];
-
-  AnswerFrameRendered(output, boost::lexical_cast<int>(frame), request);
-}
-
-
-
-
 
 static bool DisplayPerformanceWarning(OrthancPluginContext* context)
 {
@@ -561,9 +482,6 @@ extern "C"
       // Read the configuration
       OrthancPlugins::Configuration::Initialize();
 
-      // Initialize GDCM
-      OrthancPlugins::GdcmParsedDicomFile::Initialize();
-
       // Configure the DICOMweb callbacks
       if (OrthancPlugins::Configuration::GetBooleanValue("Enable", true))
       {
@@ -591,8 +509,8 @@ extern "C"
         OrthancPlugins::RegisterRestCallback<RetrieveBulkData>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/bulk/(.*)", true);
         OrthancPlugins::RegisterRestCallback<RetrieveInstanceMetadata>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/metadata", true);
         OrthancPlugins::RegisterRestCallback<RetrieveSeriesMetadata>(root + "studies/([^/]*)/series/([^/]*)/metadata", true);
-        OrthancPlugins::RegisterRestCallback<RetrieveFrames>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames", true);
-        OrthancPlugins::RegisterRestCallback<RetrieveFrames>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveAllFrames>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveSelectedFrames>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)", true);
 
         OrthancPlugins::RegisterRestCallback<ListServers>(root + "servers", true);
         OrthancPlugins::RegisterRestCallback<ListServerOperations>(root + "servers/([^/]*)", true);
@@ -609,6 +527,8 @@ extern "C"
 
         OrthancPlugins::RegisterRestCallback<GetClientInformation>(root + "info", true);
 
+        OrthancPlugins::RegisterRestCallback<RetrieveStudyRendered>(root + "studies/([^/]*)/rendered", true);
+        OrthancPlugins::RegisterRestCallback<RetrieveSeriesRendered>(root + "studies/([^/]*)/series/([^/]*)/rendered", true);
         OrthancPlugins::RegisterRestCallback<RetrieveInstanceRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/rendered", true);
         OrthancPlugins::RegisterRestCallback<RetrieveFrameRendered>(root + "studies/([^/]*)/series/([^/]*)/instances/([^/]*)/frames/([^/]*)/rendered", true);
 
