@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -24,6 +24,7 @@
 #include "Configuration.h"
 #include "DicomWebFormatter.h"
 
+#include <Core/DicomFormat/DicomMap.h>
 #include <Core/DicomFormat/DicomTag.h>
 #include <Core/Toolbox.h>
 #include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
@@ -36,41 +37,6 @@
 
 namespace
 {
-  static std::string GetOrthancTag(const Json::Value& source,
-                                   const Orthanc::DicomTag& tag,
-                                   const std::string& defaultValue)
-  {
-    const std::string s = tag.Format();
-
-    if (source.isMember(s))
-    {
-      switch (source[s].type())
-      {
-        case Json::stringValue:
-          return source[s].asString();
-
-          // The conversions below should *not* be necessary
-
-        case Json::intValue:
-          return boost::lexical_cast<std::string>(source[s].asInt());
-
-        case Json::uintValue:
-          return boost::lexical_cast<std::string>(source[s].asUInt());
-
-        case Json::realValue:
-          return boost::lexical_cast<std::string>(source[s].asFloat());
-
-        default:
-          return defaultValue;
-      }
-    }
-    else
-    {
-      return defaultValue;
-    }
-  }
-
-
   class ModuleMatcher
   {
   public:
@@ -83,62 +49,64 @@ namespace
     std::list<Orthanc::DicomTag>  includeFields_;
     bool                          includeAllFields_;
     Filters                       filters_;
+    bool                          filteredStudyInstanceUid_;
+    bool                          filteredSeriesInstanceUid_;
 
 
-    static void AddResultAttributesForLevel(std::list<Orthanc::DicomTag>& result,
+    static void AddResultAttributesForLevel(std::set<Orthanc::DicomTag>& result,
                                             Orthanc::ResourceType level)
     {
       switch (level)
       {
         case Orthanc::ResourceType_Study:
-          // http://medical.nema.org/medical/dicom/current/output/html/part18.html#table_6.7.1-2
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0020));  // Study Date
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0030));  // Study Time
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0050));  // Accession Number
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0056));  // Instance Availability
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x0061));  // Modalities in Study  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0090));  // Referring Physician's Name
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0010, 0x0010));  // Patient's Name
-          result.push_back(Orthanc::DicomTag(0x0010, 0x0020));  // Patient ID
-          result.push_back(Orthanc::DicomTag(0x0010, 0x0030));  // Patient's Birth Date
-          result.push_back(Orthanc::DicomTag(0x0010, 0x0040));  // Patient's Sex
-          result.push_back(Orthanc::DicomTag(0x0020, 0x000D));  // Study Instance UID
-          result.push_back(Orthanc::DicomTag(0x0020, 0x0010));  // Study ID
-          //result.push_back(Orthanc::DicomTag(0x0020, 0x1206));  // Number of Study Related Series  => SPECIAL CASE
-          //result.push_back(Orthanc::DicomTag(0x0020, 0x1208));  // Number of Study Related Instances  => SPECIAL CASE
+          // http://dicom.nema.org/medical/dicom/2019a/output/html/part18.html#table_6.7.1-2
+          //result.insert(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0008, 0x0020));  // Study Date
+          result.insert(Orthanc::DicomTag(0x0008, 0x0030));  // Study Time
+          result.insert(Orthanc::DicomTag(0x0008, 0x0050));  // Accession Number
+          result.insert(Orthanc::DicomTag(0x0008, 0x0056));  // Instance Availability
+          //result.insert(Orthanc::DicomTag(0x0008, 0x0061));  // Modalities in Study  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0008, 0x0090));  // Referring Physician's Name
+          result.insert(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
+          //result.insert(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0010, 0x0010));  // Patient's Name
+          result.insert(Orthanc::DicomTag(0x0010, 0x0020));  // Patient ID
+          result.insert(Orthanc::DicomTag(0x0010, 0x0030));  // Patient's Birth Date
+          result.insert(Orthanc::DicomTag(0x0010, 0x0040));  // Patient's Sex
+          result.insert(Orthanc::DicomTag(0x0020, 0x000D));  // Study Instance UID
+          result.insert(Orthanc::DicomTag(0x0020, 0x0010));  // Study ID
+          //result.insert(Orthanc::DicomTag(0x0020, 0x1206));  // Number of Study Related Series  => SPECIAL CASE
+          //result.insert(Orthanc::DicomTag(0x0020, 0x1208));  // Number of Study Related Instances  => SPECIAL CASE
           break;
 
         case Orthanc::ResourceType_Series:
-          // http://medical.nema.org/medical/dicom/current/output/html/part18.html#table_6.7.1-2a
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0060));  // Modality
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
-          result.push_back(Orthanc::DicomTag(0x0008, 0x103E));  // Series Description
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0020, 0x000E));  // Series Instance UID
-          result.push_back(Orthanc::DicomTag(0x0020, 0x0011));  // Series Number
-          //result.push_back(Orthanc::DicomTag(0x0020, 0x1209));  // Number of Series Related Instances  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0040, 0x0244));  // Performed Procedure Step Start Date
-          result.push_back(Orthanc::DicomTag(0x0040, 0x0245));  // Performed Procedure Step Start Time
-          result.push_back(Orthanc::DicomTag(0x0040, 0x0275));  // Request Attribute Sequence
+          // http://dicom.nema.org/medical/dicom/2019a/output/html/part18.html#table_6.7.1-2a
+          //result.insert(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0008, 0x0060));  // Modality
+          result.insert(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
+          result.insert(Orthanc::DicomTag(0x0008, 0x103E));  // Series Description
+          //result.insert(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0020, 0x000E));  // Series Instance UID
+          result.insert(Orthanc::DicomTag(0x0020, 0x0011));  // Series Number
+          //result.insert(Orthanc::DicomTag(0x0020, 0x1209));  // Number of Series Related Instances  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0040, 0x0244));  // Performed Procedure Step Start Date
+          result.insert(Orthanc::DicomTag(0x0040, 0x0245));  // Performed Procedure Step Start Time
+          result.insert(Orthanc::DicomTag(0x0040, 0x0275));  // Request Attribute Sequence
           break;
 
         case Orthanc::ResourceType_Instance:
-          // http://medical.nema.org/medical/dicom/current/output/html/part18.html#table_6.7.1-2b
-          //result.push_back(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0016));  // SOP Class UID
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0018));  // SOP Instance UID
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0056));  // Instance Availability
-          result.push_back(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
-          result.push_back(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL
-          result.push_back(Orthanc::DicomTag(0x0020, 0x0013));  // Instance Number
-          result.push_back(Orthanc::DicomTag(0x0028, 0x0010));  // Rows
-          result.push_back(Orthanc::DicomTag(0x0028, 0x0011));  // Columns
-          result.push_back(Orthanc::DicomTag(0x0028, 0x0100));  // Bits Allocated
-          result.push_back(Orthanc::DicomTag(0x0028, 0x0008));  // Number of Frames
+          // http://dicom.nema.org/medical/dicom/2019a/output/html/part18.html#table_6.7.1-2b
+          //result.insert(Orthanc::DicomTag(0x0008, 0x0005));  // Specific Character Set  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0008, 0x0016));  // SOP Class UID
+          result.insert(Orthanc::DicomTag(0x0008, 0x0018));  // SOP Instance UID
+          result.insert(Orthanc::DicomTag(0x0008, 0x0056));  // Instance Availability
+          result.insert(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
+          result.insert(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL
+          result.insert(Orthanc::DicomTag(0x0020, 0x0013));  // Instance Number
+          result.insert(Orthanc::DicomTag(0x0028, 0x0010));  // Rows
+          result.insert(Orthanc::DicomTag(0x0028, 0x0011));  // Columns
+          result.insert(Orthanc::DicomTag(0x0028, 0x0100));  // Bits Allocated
+          result.insert(Orthanc::DicomTag(0x0028, 0x0008));  // Number of Frames
           break;
 
         default:
@@ -152,7 +120,9 @@ namespace
       fuzzy_(false),
       offset_(0),
       limit_(0),
-      includeAllFields_(false)
+      includeAllFields_(false),
+      filteredStudyInstanceUid_(false),
+      filteredSeriesInstanceUid_(false)
     {
       std::string args;
       
@@ -214,7 +184,17 @@ namespace
           Orthanc::DicomTag tag(0, 0);
           if (OrthancPlugins::ParseTag(tag, key))
           {
-            filters_[tag] = value;
+            // The following lines are new in DICOMweb > 1.0, and
+            // allow to query against a list of multiple values
+            // http://dicom.nema.org/MEDICAL/dicom/2019a/output/chtml/part18/sect_6.7.html#sect_6.7.1.1.1
+            boost::replace_all(value, "\\", "");  // Remove backslashes from source request
+
+            // Replace commas by backslashes
+            boost::replace_all(value, ",", "\\");
+            boost::replace_all(value, "%2c", "\\");
+            boost::replace_all(value, "%2C", "\\");
+            
+            AddFilter(tag, value, false);
           }
         }
       }
@@ -233,9 +213,22 @@ namespace
     }
 
     void AddFilter(const Orthanc::DicomTag& tag,
-                   const std::string& constraint)
+                   const std::string& constraint,
+                   bool isFromPath)
     {
       filters_[tag] = constraint;
+
+      if (!isFromPath)
+      {
+        if (tag == Orthanc::DICOM_TAG_STUDY_INSTANCE_UID)
+        {
+          filteredStudyInstanceUid_ = true;
+        }
+        else if (tag == Orthanc::DICOM_TAG_SERIES_INSTANCE_UID)
+        {
+          filteredSeriesInstanceUid_ = true;
+        }
+      }
     }
 
     void Print(std::ostream& out) const 
@@ -245,6 +238,8 @@ namespace
       {
         printf("Filter [%04x,%04x] = [%s]\n", it->first.GetGroup(), it->first.GetElement(), it->second.c_str());
       }
+      printf("QIDO on StudyInstanceUID: %d\n", filteredStudyInstanceUid_);
+      printf("QIDO on SeriesInstanceUID: %d\n\n", filteredSeriesInstanceUid_);
     }
 
     void ConvertToOrthanc(Json::Value& result,
@@ -270,8 +265,13 @@ namespace
           throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
 
+      bool caseSensitive;
+      if (OrthancPlugins::Configuration::LookupBooleanValue(caseSensitive, "QidoCaseSensitive"))
+      {
+        result["CaseSensitive"] = caseSensitive;
+      }
+
       result["Expand"] = false;
-      result["CaseSensitive"] = OrthancPlugins::Configuration::GetBooleanValue("QidoCaseSensitive", true);
       result["Query"] = Json::objectValue;
       result["Limit"] = limit_;
       result["Since"] = offset_;
@@ -292,36 +292,63 @@ namespace
     }
 
 
-    void ComputeDerivedTags(Filters& target,
+    void ComputeDerivedTags(Orthanc::DicomMap& target,
+                            std::string& someInstance,
                             Orthanc::ResourceType level,
                             const std::string& resource) const
     {
-      target.clear();
-
+      static const char* const INSTANCES = "Instances";      
+      static const char* const MAIN_DICOM_TAGS = "MainDicomTags";
+      static const char* const MODALITY = "Modality";
+      
       switch (level)
       {
+        case Orthanc::ResourceType_Instance:
+          someInstance = resource;
+          break;
+
         case Orthanc::ResourceType_Study:
         {
-          Json::Value series, instances;
+          Json::Value series;
           if (OrthancPlugins::RestApiGet(series, "/studies/" + resource + "/series?expand", false) &&
-              OrthancPlugins::RestApiGet(instances, "/studies/" + resource + "/instances", false))
+              series.type() == Json::arrayValue)
           {
-            target[Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES] = 
-              boost::lexical_cast<std::string>(series.size());
-            target[Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES] = 
-              boost::lexical_cast<std::string>(instances.size());
-
-            // Collect the Modality of all the child series
+            // Collect the Modality of all the child series, and 
             std::set<std::string> modalities;
+            unsigned int countInstances = 0;
+            
             for (Json::Value::ArrayIndex i = 0; i < series.size(); i++)
             {
-              if (series[i].isMember("MainDicomTags") &&
-                  series[i]["MainDicomTags"].isMember("Modality"))
+              if (series[i].type() == Json::objectValue)
               {
-                modalities.insert(series[i]["MainDicomTags"]["Modality"].asString());
+                if (series[i].isMember(MAIN_DICOM_TAGS) &&
+                    series[i][MAIN_DICOM_TAGS].type() == Json::objectValue &&
+                    series[i][MAIN_DICOM_TAGS].isMember(MODALITY) &&
+                    series[i][MAIN_DICOM_TAGS][MODALITY].type() == Json::stringValue)
+                {
+                  modalities.insert(series[i][MAIN_DICOM_TAGS][MODALITY].asString());
+                }
+                
+                if (series[i].isMember(INSTANCES) &&
+                    series[i][INSTANCES].type() == Json::arrayValue)
+                {
+                  if (series[i][INSTANCES].size() > 0 &&
+                      series[i][INSTANCES][0].type() == Json::stringValue)
+                  {
+                    someInstance = series[i][INSTANCES][0].asString();
+                  }
+                  
+                  countInstances += series[i][INSTANCES].size();
+                }
               }
             }
 
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES,
+                            boost::lexical_cast<std::string>(series.size()), false);
+            
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES,
+                            boost::lexical_cast<std::string>(countInstances), false);
+            
             std::string s;
             for (std::set<std::string>::const_iterator 
                    it = modalities.begin(); it != modalities.end(); ++it)
@@ -334,13 +361,13 @@ namespace
               s += *it;
             }
 
-            target[Orthanc::DICOM_TAG_MODALITIES_IN_STUDY] = s;
+            target.SetValue(Orthanc::DICOM_TAG_MODALITIES_IN_STUDY, s, false);
           }
           else
           {
-            target[Orthanc::DICOM_TAG_MODALITIES_IN_STUDY] = "";
-            target[Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES] = "0";
-            target[Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES] = "0"; 
+            target.SetValue(Orthanc::DICOM_TAG_MODALITIES_IN_STUDY, "", false);
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES, "0", false);
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES, "0", false);
           }
 
           break;
@@ -348,34 +375,49 @@ namespace
 
         case Orthanc::ResourceType_Series:
         {
-          Json::Value instances;
-          if (OrthancPlugins::RestApiGet(instances, "/series/" + resource + "/instances", false))
+          Json::Value series;
+          if (OrthancPlugins::RestApiGet(series, "/series/" + resource, false) &&
+              series.type() == Json::objectValue &&
+              series.isMember(INSTANCES) &&
+              series[INSTANCES].type() == Json::arrayValue)
           {
+            if (series[INSTANCES].size() > 0 &&
+                series[INSTANCES][0].type() == Json::stringValue)
+            {
+              someInstance = series[INSTANCES][0].asString();
+            }
+            
             // Number of Series Related Instances
-            target[Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES] = 
-              boost::lexical_cast<std::string>(instances.size());
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES, 
+                            boost::lexical_cast<std::string>(series[INSTANCES].size()), false);
           }
           else
           {
-            target[Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES] = "0";
+            // Should never happen
+            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES, "0", false);
           }
 
           break;
         }
 
         default:
-          break;
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
     }                              
 
 
-    void ExtractFields(Json::Value& result,
-                       const Json::Value& source,
+    void ExtractFields(Orthanc::DicomMap& result,
+                       const Orthanc::DicomMap& source,
                        const std::string& wadoBase,
                        Orthanc::ResourceType level) const
     {
-      result = Json::objectValue;
-      std::list<Orthanc::DicomTag> fields = includeFields_;
+      std::set<Orthanc::DicomTag> fields;
+
+      for (std::list<Orthanc::DicomTag>::const_iterator
+             it = includeFields_.begin(); it != includeFields_.end(); ++it)
+      {
+        fields.insert(*it);
+      }
 
       // The list of attributes for this query level
       AddResultAttributesForLevel(fields, level);
@@ -384,58 +426,54 @@ namespace
       for (Filters::const_iterator it = filters_.begin();
            it != filters_.end(); ++it)
       {
-        fields.push_back(it->first);
+        fields.insert(it->first);
       }
 
       // For instances and series, add all Study-level attributes if
       // {StudyInstanceUID} is not specified.
-      if ((level == Orthanc::ResourceType_Instance  || level == Orthanc::ResourceType_Series) 
-          && filters_.find(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID) == filters_.end()
-        )
+      if (!filteredStudyInstanceUid_ &&
+          (level == Orthanc::ResourceType_Instance ||
+           level == Orthanc::ResourceType_Series))
       {
         AddResultAttributesForLevel(fields, Orthanc::ResourceType_Study);
       }
 
       // For instances, add all Series-level attributes if
       // {SeriesInstanceUID} is not specified.
-      if (level == Orthanc::ResourceType_Instance
-          && filters_.find(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID) == filters_.end()
-        )
+      if (!filteredSeriesInstanceUid_ &&
+          level == Orthanc::ResourceType_Instance)
       {
         AddResultAttributesForLevel(fields, Orthanc::ResourceType_Series);
       }
 
       // Copy all the required fields to the target
-      for (std::list<Orthanc::DicomTag>::const_iterator
+      for (std::set<Orthanc::DicomTag>::const_iterator
              it = fields.begin(); it != fields.end(); ++it)
       {
-        // Complies to the JSON produced internally by Orthanc
-        char tag[16];
-        sprintf(tag, "%04x,%04x", it->GetGroup(), it->GetElement());
-        
-        if (source.isMember(tag))
+        std::string value;
+        if (source.LookupStringValue(value, *it, false /* no binary */))
         {
-          result[tag] = source[tag];
+          result.SetValue(*it, value, false);
         }
       }
 
       // Set the retrieve URL for WADO-RS
-      std::string url = (wadoBase + "studies/" + 
-                         GetOrthancTag(source, Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, ""));
+      std::string url = (wadoBase + "studies/" +
+                         source.GetStringValue(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, "", false));
 
       if (level == Orthanc::ResourceType_Series || 
           level == Orthanc::ResourceType_Instance)
       {
-        url += "/series/" + GetOrthancTag(source, Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, "");
+        url += "/series/" + source.GetStringValue(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, "", false);
       }
 
       if (level == Orthanc::ResourceType_Instance)
       {
-        url += "/instances/" + GetOrthancTag(source, Orthanc::DICOM_TAG_SOP_INSTANCE_UID, "");
+        url += "/instances/" + source.GetStringValue(Orthanc::DICOM_TAG_SOP_INSTANCE_UID, "", false);
       }
     
       static const Orthanc::DicomTag DICOM_TAG_RETRIEVE_URL(0x0008, 0x1190);
-      result[DICOM_TAG_RETRIEVE_URL.Format()] = url;
+      result.SetValue(DICOM_TAG_RETRIEVE_URL, url, false);
     }
   };
 }
@@ -450,6 +488,8 @@ static void ApplyMatcher(OrthancPluginRestOutput* output,
   Json::Value find;
   matcher.ConvertToOrthanc(find, level);
 
+  LOG(INFO) << "Body of the call from QIDO-RS to /tools/find: " << find.toStyledString();
+  
   std::string body;
 
   {
@@ -464,65 +504,38 @@ static void ApplyMatcher(OrthancPluginRestOutput* output,
     throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
   }
 
-  typedef std::list< std::pair<std::string, std::string> > ResourcesAndInstances;
-
-  ResourcesAndInstances resourcesAndInstances;
-  std::string root = (level == Orthanc::ResourceType_Study ? "/studies/" : "/series/");
-    
-  for (Json::Value::ArrayIndex i = 0; i < resources.size(); i++)
-  {
-    const std::string resource = resources[i].asString();
-
-    if (level == Orthanc::ResourceType_Study ||
-        level == Orthanc::ResourceType_Series)
-    {
-      // Find one child instance of this resource
-      Json::Value tmp;
-      if (OrthancPlugins::RestApiGet(tmp, root + resource + "/instances", false) &&
-          tmp.type() == Json::arrayValue &&
-          tmp.size() > 0)
-      {
-        resourcesAndInstances.push_back(std::make_pair(resource, tmp[0]["ID"].asString()));
-      }
-    }
-    else
-    {
-      resourcesAndInstances.push_back(std::make_pair(resource, resource));
-    }
-  }
-  
   std::string wadoBase = OrthancPlugins::Configuration::GetBaseUrl(request);
 
   OrthancPlugins::DicomWebFormatter::HttpWriter writer(
     output, OrthancPlugins::Configuration::IsXmlExpected(request));
 
   // Fix of issue #13
-  for (ResourcesAndInstances::const_iterator
-         it = resourcesAndInstances.begin(); it != resourcesAndInstances.end(); ++it)
+  for (Json::Value::ArrayIndex i = 0; i < resources.size(); i++)
   {
+    const std::string resource = resources[i].asString();
+
+    Orthanc::DicomMap derivedTags;
+    std::string someInstance;
+    matcher.ComputeDerivedTags(derivedTags, someInstance, level, resource);
+    
     Json::Value tags;
-    if (OrthancPlugins::RestApiGet(tags, "/instances/" + it->second + "/tags?short", false))
+    if (!someInstance.empty() &&
+        OrthancPlugins::RestApiGet(tags, "/instances/" + someInstance + "/tags", false))
     {
+      Orthanc::DicomMap source;
+      source.FromDicomAsJson(tags);
+
       std::string wadoUrl = OrthancPlugins::Configuration::GetWadoUrl(
-        wadoBase, 
-        GetOrthancTag(tags, Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, ""),
-        GetOrthancTag(tags, Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, ""),
-        GetOrthancTag(tags, Orthanc::DICOM_TAG_SOP_INSTANCE_UID, ""));
+        wadoBase,
+        source.GetStringValue(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, "", false),
+        source.GetStringValue(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, "", false),
+        source.GetStringValue(Orthanc::DICOM_TAG_SOP_INSTANCE_UID, "", false));
 
-      Json::Value result;
-      matcher.ExtractFields(result, tags, wadoBase, level);
+      Orthanc::DicomMap target;
+      target.Assign(derivedTags);
+      matcher.ExtractFields(target, source, wadoBase, level);
 
-      // Inject the derived tags
-      ModuleMatcher::Filters derivedTags;
-      matcher.ComputeDerivedTags(derivedTags, level, it->first);
-
-      for (ModuleMatcher::Filters::const_iterator
-             tag = derivedTags.begin(); tag != derivedTags.end(); ++tag)
-      {
-        result[tag->first.Format()] = tag->second;
-      }
-
-      writer.AddJson(result);
+      writer.AddOrthancMap(target);
     }
   }
 
@@ -562,7 +575,7 @@ void SearchForSeries(OrthancPluginRestOutput* output,
     if (request->groupsCount == 1)
     {
       // The "StudyInstanceUID" is provided by the regular expression
-      matcher.AddFilter(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, request->groups[0]);
+      matcher.AddFilter(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, request->groups[0], true);
     }
 
     ApplyMatcher(output, request, matcher, Orthanc::ResourceType_Series);
@@ -586,13 +599,13 @@ void SearchForInstances(OrthancPluginRestOutput* output,
         request->groupsCount == 2)
     {
       // The "StudyInstanceUID" is provided by the regular expression
-      matcher.AddFilter(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, request->groups[0]);
+      matcher.AddFilter(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, request->groups[0], true);
     }
 
     if (request->groupsCount == 2)
     {
       // The "SeriesInstanceUID" is provided by the regular expression
-      matcher.AddFilter(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, request->groups[1]);
+      matcher.AddFilter(Orthanc::DICOM_TAG_SERIES_INSTANCE_UID, request->groups[1], true);
     }
 
     ApplyMatcher(output, request, matcher, Orthanc::ResourceType_Instance);
