@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -24,8 +24,22 @@
 #include "Configuration.h"
 
 
+bool LocateStudy(OrthancPluginRestOutput* output,
+                 std::string& uri,
+                 std::string& studyInstanceUid,
+                 const OrthancPluginHttpRequest* request);
+
+bool LocateSeries(OrthancPluginRestOutput* output,
+                  std::string& uri,
+                  std::string& studyInstanceUid,
+                  std::string& seriesInstanceUid,
+                  const OrthancPluginHttpRequest* request);
+
 bool LocateInstance(OrthancPluginRestOutput* output,
                     std::string& uri,
+                    std::string& studyInstanceUid,
+                    std::string& seriesInstanceUid,
+                    std::string& sopInstanceUid,
                     const OrthancPluginHttpRequest* request);
 
 void RetrieveDicomStudy(OrthancPluginRestOutput* output,
@@ -56,6 +70,26 @@ void RetrieveBulkData(OrthancPluginRestOutput* output,
                       const char* url,
                       const OrthancPluginHttpRequest* request);
 
-void RetrieveFrames(OrthancPluginRestOutput* output,
-                    const char* url,
-                    const OrthancPluginHttpRequest* request);
+void RetrieveAllFrames(OrthancPluginRestOutput* output,
+                       const char* url,
+                       const OrthancPluginHttpRequest* request);
+
+void RetrieveSelectedFrames(OrthancPluginRestOutput* output,
+                            const char* url,
+                            const OrthancPluginHttpRequest* request);
+
+void RetrieveInstanceRendered(OrthancPluginRestOutput* output,
+                              const char* url,
+                              const OrthancPluginHttpRequest* request);
+
+void RetrieveFrameRendered(OrthancPluginRestOutput* output,
+                           const char* url,
+                           const OrthancPluginHttpRequest* request);
+
+void RetrieveSeriesRendered(OrthancPluginRestOutput* output,
+                            const char* url,
+                            const OrthancPluginHttpRequest* request);
+
+void RetrieveStudyRendered(OrthancPluginRestOutput* output,
+                           const char* url,
+                           const OrthancPluginHttpRequest* request);
